--  title: SPACE, 0.10
--     by: Matthew Sephton, Tektonix Software 
-- e-mail: u5ms@csc.liv.ac.uk
--    www: http://www.csc.liv.ac.uk/~u5ms
--    ed.: 0.10, slight tweaking of code
--
-- Here's my first dabble in Euphoria, just to see what sort of speed it can
-- get up.  Use the numeric pad to control your flight through space (NUM LOCK
-- should be off) and feel free to change the constant parameters at the
-- beginning.  The speed is only really affected by those marked with a *.
-- At higher resolutions apparent speed is less.
-- 
-- FUTURE: Totally free movement, planets, a spaceship, page swapping, a game?

without type_check

include graphics.e

constant GRAPHICS_MODE = 19             -- 320x200 256 colours, see graphics.e

constant POPULATION = 200               -- number of stars *
constant LEVELS = 6                     -- parallax levels (mults of 3 best)
constant MESSAGE = "E u p h o r i a !"  -- message to display *

sequence stars, speeds, colours, dir
integer max_x, max_y, text_x, text_y
sequence vc

procedure init()
-- initialize global variables
    if graphics_mode(GRAPHICS_MODE) then
       puts(1, "Needs at least VGA graphics!\n")
       abort(1)
    end if
    vc = video_config()

    -- screen edge limits
    max_x = vc[VC_XPIXELS]
    max_y = vc[VC_YPIXELS]

    -- blue message in middle of screen
    text_color(BRIGHT_BLUE)
    text_x = floor((vc[VC_COLUMNS] / 2) - floor(length(MESSAGE) / 2)) + 1
    text_y = floor(vc[VC_LINES] / 2) + 1

    stars = {}
    speeds = {}
    colours = {}
    dir = {1, 0}    -- stars going right initially
end procedure

procedure space()
-- main routine
    sequence old_position
    integer x, y, s
    atom k

    init()

    -- main loop
    while 1 do
	k = get_key()
	if k != -1 then         -- key pressed...
	   if k = 27 then       -- ESC, quits
	      exit
	   elsif k = 327 then   -- HOME, stars go down & left
	      dir = {1, 1}
	   elsif k = 328 then   -- UP, stars go down
	      dir = {0, 1}
	   elsif k = 329 then   -- PG UP, stars go down & right
	      dir = {-1, 1}
	   elsif k = 331 then   -- LEFT, stars go right
	      dir = {1, 0}
	   elsif k = 332 then   -- 5, stars stay still
	      dir = {0, 0}
	   elsif k = 333 then   -- RIGHT, stars go left
	      dir = {-1, 0}
	   elsif k = 335 then   -- END, stars go up & left
	      dir = {1, -1}
	   elsif k = 336 then   -- DOWN, stars go up
	      dir = {0, -1}
	   elsif k = 337 then   -- PG DN, stars go up & right
	      dir = {-1, -1}
	   end if
	end if

	-- create starfield
	if length(stars) <= POPULATION then
	    -- add a new star
	    x = rand(max_x)                             -- random position
	    y = rand(max_y)
	    stars = append(stars, {x, y})               
	    s = rand(LEVELS)                            -- random level
	    speeds = append(speeds, s)

	    if s > floor(LEVELS * 2/3) then             -- top (white)
	       colours = append(colours, BRIGHT_WHITE)
	    elsif s <= floor(LEVELS / 3) then           -- bottom (grey)
	       colours = append(colours, GRAY)
	    else                                        -- middle (light grey)
	       colours = append(colours, WHITE)
	    end if
	end if

	-- move all the stars
	for i = 1 to length(stars) do
	    old_position = stars[i]
	    stars[i][1] = stars[i][1] + (dir[1] * speeds[i]) 
	    stars[i][2] = stars[i][2] + (dir[2] * speeds[i]) 

	    -- check for off screen location
	    if (dir[1] = -1 and stars[i][1] < 0) then
	       stars[i][1] = max_x + stars[i][1]
	    elsif (dir[1] = 1 and stars[i][1] > vc[VC_XPIXELS]) then
	       stars[i][1] = vc[VC_XPIXELS] - stars[i][1]
	    elsif (dir[2] = -1 and stars[i][2] < 0) then
	       stars[i][2] = max_y + stars[i][2]
	    elsif (dir[2] = 1 and stars[i][2] > vc[VC_YPIXELS]) then
	       stars[i][2] = vc[VC_YPIXELS] - stars[i][2]
	    end if

	    -- blank out previous position
	    pixel(BLACK, old_position)
	    -- draw at new position
	    pixel(colours[i], stars[i])
	end for

	-- message
	position(text_y, text_x)
	puts(1, MESSAGE)
    end while

    -- revert to old graphics mode
    if graphics_mode(-1) then
    end if
end procedure

-- start program
space()

