--  title: SPACE 2
--     by: Matthew Sephton                          
-- e-mail: u5ms@csc.liv.ac.uk
--    www: http://www.csc.liv.ac.uk/~u5ms
--
-- Variation on the SPACE theme.
--
-- FUTURE: None really, it's not as versatile as the first SPACE.

without type_check

include graphics.e

constant GRAPHICS_MODE = 19             -- 320x200 256 colours, see graphics.e

constant POPULATION = 300               -- number of stars *
constant LEVELS = 3                     -- parallax levels (mults of 3 best)
constant MESSAGE = "E u p h o r i a !"  -- message to display *

sequence stars, speeds, colours, dir
integer max_x, max_y, text_x, text_y
sequence vc

procedure init()
-- initialize global variables
    if graphics_mode(GRAPHICS_MODE) then
       puts(1, "Needs at least VGA graphics!\n")
       abort(1)
    end if
    vc = video_config()

    -- screen edge limits
    max_x = floor(vc[VC_XPIXELS] / 2)
    max_y = floor(vc[VC_YPIXELS] / 2)

    -- blue message in middle of screen
    text_color(BRIGHT_BLUE)
    text_x = floor((vc[VC_COLUMNS] / 2) - floor(length(MESSAGE) / 2)) + 1
    text_y = floor(vc[VC_LINES] / 2) + 1

    stars = {}
    speeds = {}
    colours = {}
    dir = {}
end procedure

procedure space()
-- main routine
    sequence old_position
    atom dx, dy, s
    atom k

    init()

    -- main loop
    while 1 do
	k = get_key()
	if k != -1 then         -- key pressed...
	   if k = 27 then       -- ESC, quits
	      exit
	   end if
	end if

	-- create starfield
	if length(stars) <= POPULATION then
	    -- add a new star
	    stars = append(stars, {max_x, max_y})       -- centre start
	    dx = (rand(max_x) - max_x / 2) / (max_x / 2)  -- random x
	    dy = (rand(max_y) - max_y / 2) / (max_y / 2)  -- random y
	    dir = append(dir, {dx, dy})                 -- random direction
	    s = rand(LEVELS)                            -- random level
	    speeds = append(speeds, s)

	    if s > floor(LEVELS * 2/3) then             -- top (white)
	       colours = append(colours, BRIGHT_WHITE)
	    elsif s <= floor(LEVELS / 3) then           -- bottom (grey)
	       colours = append(colours, GRAY)
	    else                                        -- middle (light grey)
	       colours = append(colours, WHITE)
	    end if
	end if

	-- move all the stars
	for i = 1 to length(stars) do
	    old_position = stars[i]
	    stars[i][1] = stars[i][1] + (dir[i][1] * speeds[i])
	    stars[i][2] = stars[i][2] + (dir[i][2] * speeds[i])

	    -- check for off screen location
	    if stars[i][1] < 0 or stars[i][1] > vc[VC_XPIXELS] or
	       stars[i][2] < 0 or stars[i][2] > vc[VC_YPIXELS] then
	       stars[i][1] = max_x
	       stars[i][2] = max_y
	    end if

	    -- blank out previous position
	    pixel(BLACK, old_position)
	    -- draw at new position
	    pixel(colours[i], stars[i])
	end for

	-- message
	position(text_y, text_x)
	puts(1, MESSAGE)
    end while

    -- revert to old graphics mode
    if graphics_mode(-1) then
    end if
end procedure

-- start program
space()

